from social.tests.test_exceptions import *
from social.tests.test_pipeline import *
from social.tests.test_storage import *
from social.tests.test_utils import *
from social.tests.actions.test_associate import *
from social.tests.actions.test_disconnect import *
from social.tests.actions.test_login import *
from social.tests.backends.test_amazon import *
from social.tests.backends.test_angel import *
from social.tests.backends.test_behance import *
from social.tests.backends.test_bitbucket import *
from social.tests.backends.test_box import *
from social.tests.backends.test_broken import *
from social.tests.backends.test_coinbase import *
from social.tests.backends.test_dailymotion import *
from social.tests.backends.test_disqus import *
from social.tests.backends.test_dropbox import *
from social.tests.backends.test_dummy import *
from social.tests.backends.test_email import *
from social.tests.backends.test_evernote import *
from social.tests.backends.test_facebook import *
from social.tests.backends.test_fitbit import *
from social.tests.backends.test_flickr import *
from social.tests.backends.test_foursquare import *
from social.tests.backends.test_google import *
from social.tests.backends.test_instagram import *
from social.tests.backends.test_linkedin import *
from social.tests.backends.test_live import *
from social.tests.backends.test_livejournal import *
from social.tests.backends.test_mixcloud import *
from social.tests.backends.test_podio import *
from social.tests.backends.test_readability import *
from social.tests.backends.test_reddit import *
from social.tests.backends.test_skyrock import *
from social.tests.backends.test_soundcloud import *
from social.tests.backends.test_stackoverflow import *
from social.tests.backends.test_steam import *
from social.tests.backends.test_stocktwits import *
from social.tests.backends.test_stripe import *
from social.tests.backends.test_thisismyjam import *
from social.tests.backends.test_tripit import *
from social.tests.backends.test_tumblr import *
from social.tests.backends.test_twitter import *
from social.tests.backends.test_username import *
from social.tests.backends.test_utils import *
from social.tests.backends.test_vk import *
from social.tests.backends.test_xing import *
from social.tests.backends.test_yahoo import *
from social.tests.backends.test_yammer import *
from social.tests.backends.test_yandex import *
