"""
python-social-auth application, allows OpenId or OAuth user
registration/authentication just adding a few configurations.
"""
version = (0, 1, 27)
extra = '-hh11'
__version__ = '.'.join(map(str, version)) + extra
